    <!-- Modal -->
    <div class="modal fade" id="dialogAbout" tabindex="-1"
         role="dialog"
         aria-labelledby="myModalDialog"
         aria-hidden="true">
        <div class="modal-dialog">
            <div class="modal-content">
                <div class="modal-header">
                    <button type="button" class="close" data-dismiss="modal"
                            aria-label="Close">
                        <span aria-hidden="true">&times;</span></button>
                    <h4 class="modal-title" id="myModalDialog" translate>
                        About privacyIDEA Web Client</h4>
                </div>
                <div class="modal-body">
                    <h4 translate>privacyIDEA Single Page Web Client</h4>

                    <p translate>
                        This is a Single Page Application client that acts as a
                        frontend for
                        privacyIDEA authentication backend.
                    </p>

                    <p>
                        <translate>Licensed under</translate>
                        <a href="http://www.gnu.org/licenses/agpl-3.0.en.html"
                           target="license">
                            AGPLv3</a>.<br>
                        (c) 2015, Cornelius Kölbel,
                        <a href="mailtto:cornelius.koelbel@netknights.it">
                            cornelius.koelbel@netknights.it</a>
                    </p>
                    <h4 translate>privacyIDEA authentication backend</h4>
                    <p translate>
                    The privacyIDEA authentication backend is licensed under the
                    AGPLv3.
                    Some components may be part to other licenses.
                    privacyIDEA was originally forked from LinOTP, which was
                    licensed
                    by LSE Leading Security Experts GmbH.
                    For detailed information please take a look at the
                    <a href="http://github.com/privacyidea/privacyidea"
                       target="github">backend
                        service itself</a>.
                    </p>
                </div>
                <div class="modal-footer">
                    <button type="button" class="btn btn-default"
                            data-dismiss="modal" translate>Close
                    </button>
                </div>
            </div>
        </div>
    </div>
