        <accordion close-others="oneAtATime">
            <accordion-group heading="{{ 'Registration Code'|translate }}">
                {{ enrolledToken.registrationcode }}
            </accordion-group>
        </accordion>

        <p class="help-block" translate>
            You may pass this registration code to the user.
        </p>

