<div class="row">
    <div class="col-sm-6">
        <img width="{{ qrCodeWidth }}" src="{{ enrolledToken.googleurl.img }}"/>
    </div>
    <div class="col-sm-6" ng-hide="$state.includes('token.wizard')">
        <accordion close-others="oneAtATime">
            <accordion-group heading="{{ 'The OTP key'|translate }}">
                {{ enrolledToken.otpkey.value }}
            </accordion-group>
        </accordion>

        <p translate>
            Click <a href="{{ enrolledToken.googleurl.value }}">here</a> or scan
            the QR Code, if you
            want to add the Token to your Google Authenticator.
        </p>
        <p translate>
            The QR Code contains the secret key of your token. You need to
            protect it. <b>If you are in doubt, that someone else saw it, please
            regenerate the QR Code, when noone else is watching.</b>
        </p>
        <button ng-click="regenerateToken()" class="btn btn-warning" translate>
            Regenerate QR Code
        </button>
    </div>
</div>
