<p class="help-block" translate>
    The SMS Token sends an OTP value to the mobile phone of the user.
</p>

<h4 translate>Token data</h4>

<div ng-hide="form['sms.Provider']" translate class="bg-danger">
    For SMS tokens to work properly you need to set the
    <a ui-sref="config.tokens({tokentype:'sms'})">SMS Token Config</a>!
</div>

<div class="form-group">
    <label for="phone" translate>Phone number</label>
    <input type="text"
           autofocus
           class="form-control"
           ng-model="form.phone" name="phone" />
    <div>
        <button class="btn btn-default btn-xs" ng-repeat="phone in phone_list"
                ng-click="form.phone = phone;">
            <span class="glyphicon glyphicon-phone"></span>
            {{ phone }}
        </button>
    </div>
    <label for="description" translate>Description</label>
    <input type="text" class="form-control"
            placeholder="{{ 'Some nice words...'|translate }}"
            ng-model="form.description" />
</div>

