


<ng-include src="'{{ instance }}/static/components/dialogs/views/dialog.about.html'">
</ng-include>

<ng-include src="'{{ instance }}/static/components/dialogs/views/dialog.lock.html'">
</ng-include>

<ng-include src="'{{ instance }}/static/components/dialogs/views/dialog.autocreate_realm.html'">
</ng-include>

<ng-include src="'{{ instance }}/static/components/dialogs/views/dialog.welcome.html'">
</ng-include>

<!--load all javascripts after the HTML part-->
<script src="{{ instance }}/static/contrib/js/angular.js"></script>
<script src="{{ instance }}/static/contrib/js/ui-bootstrap-tpls-0.13.0.js"></script>
<script src="{{ instance }}/static/contrib/js/jquery-1.11.3.js"></script>
<script src="{{ instance }}/static/contrib/js/bootstrap.js"></script>
<script src="{{ instance }}/static/contrib/js/angular-multi-select.js"></script>
<script src="{{ instance }}/static/contrib/js/angular-file-upload.js"></script>
<script src="{{ instance }}/static/contrib/js/hotkeys.js"></script>
<script src="{{ instance }}/static/contrib/js/ngmodules/angular-route.js"></script>
<script src="{{ instance }}/static/contrib/js/ngmodules/angular-ui-router.js"></script>
<script src="{{ instance }}/static/contrib/js/ngmodules/angular-animate.js"></script>
<script src="{{ instance }}/static/contrib/js/ngmodules/angular-idle.js"></script>
<script src="{{ instance }}/static/contrib/js/ngmodules/angular-gettext.js"></script>
<script src="{{ instance }}/static/components/translation/translations.js"></script>
<script src="{{ instance }}/static/app.js"></script>
<script src="{{ instance }}/static/components/directives/controllers/directives.js"></script>
<script src="{{ instance }}/static/components/login/factories/auth.js"></script>
<script src="{{ instance }}/static/components/login/factories/u2f.js"></script>
<script src="{{ instance }}/static/components/login/states/states.js"></script>
<script src="{{ instance }}/static/components/login/controllers/loginControllers.js"></script>
<script src="{{ instance }}/static/components/token/factories/token.js"></script>
<script src="{{ instance }}/static/components/token/factories/validate.js"></script>
<script src="{{ instance }}/static/components/token/states/states.js"></script>
<script src="{{ instance }}/static/components/token/controllers/tokenControllers.js"></script>
<script src="{{ instance }}/static/components/token/controllers/tokenDetailController.js"></script>
<script src="{{ instance }}/static/components/token/controllers/tokenLostController.js"></script>
<script src="{{ instance }}/static/components/token/controllers/tokenGetSerialController.js"></script>
<script src="{{ instance }}/static/components/token/controllers/tokenChallengesController.js"></script>
<script src="{{ instance }}/static/components/user/factories/user.js"></script>
<script src="{{ instance }}/static/components/user/states/states.js"></script>
<script src="{{ instance }}/static/components/user/controllers/userControllers.js"></script>
<script src="{{ instance }}/static/components/audit/factories/audit.js"></script>
<script src="{{ instance }}/static/components/audit/states/states.js"></script>
<script src="{{ instance }}/static/components/audit/controllers/auditControllers.js"></script>
<script src="{{ instance }}/static/components/components/factories/component.js"></script>
<script src="{{ instance }}/static/components/components/states/states.js"></script>
<script src="{{ instance }}/static/components/components/controllers/componentControllers.js"></script>
<script src="{{ instance }}/static/components/config/factories/config.js"></script>
<script src="{{ instance }}/static/components/config/states/states.js"></script>
<script src="{{ instance }}/static/components/config/controllers/configControllers.js"></script>
<script src="{{ instance }}/static/components/config/controllers/eventController.js"></script>
<script src="{{ instance }}/static/components/config/controllers/ldapMachineResolverController.js"></script>
<script src="{{ instance }}/static/components/config/controllers/smtpServerController.js"></script>
<script src="{{ instance }}/static/components/config/controllers/radiusServerController.js"></script>
<script src="{{ instance }}/static/components/config/controllers/privacyideaServerController.js"></script>
<script src="{{ instance }}/static/components/config/controllers/smsgatewayController.js"></script>
<script src="{{ instance }}/static/components/config/controllers/system.addons.js"></script>
<script src="{{ instance }}/static/components/machine/factories/machine.js"></script>
<script src="{{ instance }}/static/components/machine/states/states.js"></script>
<script src="{{ instance }}/static/components/machine/controllers/machineController.js"></script>
<script src="{{ instance }}/static/components/machine/controllers/machineDetailsController.js"></script>
<script src="{{ instance }}/static/components/register/states/states.js"></script>
<script src="{{ instance }}/static/components/register/controllers/registerControllers.js"></script>
<script src="{{ instance }}/static/components/register/factories/registerFactory.js"></script>
<script src="{{ instance }}/static/components/recovery/states/states.js"></script>
<script src="{{ instance }}/static/components/recovery/controllers/recoveryControllers.js"></script>
<script src="{{ instance }}/static/components/recovery/factories/recoveryFactory.js"></script>
<script src="{{ instance }}/static/components/filters/filters.js"></script>
<script src="{{ instance }}/static/contrib/js/angular-inform.js"></script>
<script src="{{ instance }}/static/contrib/js/u2f-api.js"></script>
</body>
</html>
