<!--
This is the directive for assigning tokens to users.
-->
<div class="form-group">
    <label for="serial" translate>Serial</label>
    <input name="serial" type="text" ng-model="newTokenObject.serial"
           autocomplete="off"
           placeholder="{{ 'start typing a serial number of a token that is notassigned, yet.'|translate}} "
           ng-keypress="toggleLoadSerials($event.which==13)"
           typeahead-wait-ms="100"
           typeahead-focus-first="false"
           typeahead="serial for serial in loadSerials($viewValue) | filter: $viewValue"
           typeahead-loading="loadingSerials" class="form-control" >
    <i ng-show="loadingSerials" class="glyphicon glyphicon-refresh"
            translate>Loading serials...</i>
</div>
<div class="form-group">
    <!-- we could add a checkRight("enrollpin") here,
        but this would not be quite accurate, since this directive is
        also used during assigning and not only during enrolling!
        -->
    <label for="otppin" translate>PIN</label>
    <input name="otppin" ng-model="newTokenObject.pin"
            type=password class="form-control"
            euqals="{{pin2}}"
            placeholder="{{ 'Type a password'|translate }}">
    <input name="otppin2" ng-model="pin2"
            type=password class="form-control"
            equals="{{newTokenObject.pin}}"
            placeholder="{{ 'Repeat password'|translate }}">
</div>
