<h1>{{ enrolledToken.serial }}</h1>
<p>
Please use the OTP values of your paper token in order one after the other.
You may scratch of or otherwise mark used values.
</p>

<table class="table table-bordered table-striped
                                ">
                            <thead>
                            <tr>
                                <th translate>#</th>
                                <th>pw</th>
                                <th translate>#</th>
                                <th>pw</th>
                                <th translate>#</th>
                                <th>pw</th>
                                <th translate>#</th>
                                <th>wp</th>
                                <th translate>#</th>
                                <th>pw</th>
                            </tr>
                            </thead>
                            <tbody>
                            <tr ng-repeat="key in otp_rows">
                                <td>{{ $index }}</td>
                                <td>{{ enrolledToken.otps[$index] }}</td>
                                <td>{{ $index+(1*otp_row_count) }}</td>
                                <td>{{ enrolledToken.otps[$index+
                                        (1*otp_row_count)] }}</td>
                                <td>{{ $index+(2*otp_row_count) }}</td>
                                <td>{{ enrolledToken.otps[$index+
                                        (2*otp_row_count)] }}</td>
                                <td>{{ $index+(3*otp_row_count) }}</td>
                                <td>{{ enrolledToken.otps[$index+
                                        (3*otp_row_count)] }}</td>
                                <td>{{ $index+(4*otp_row_count) }}</td>
                                <td>{{ enrolledToken.otps[$index+
                                        (4*otp_row_count)] }}</td>
                            </tr>
                            </tbody>
                        </table>
