{% include 'header.html' %}

<div class="navbar navbar-default navbar-fixed-top">
    <div class="container">
        <a class="navbar-brand" href="#">privacyIDEA Certificate Request</a>
    </div>
</div>

<div class="container">

<form action="{{ instance }}/certificate/enroll" method="post">

    <div class="form-group">
        <label>CA Connector: {{ ca }}</label>
    </div>

    <input type="hidden" name="authtoken" value="{{ authtoken }}">

    <div class="form-group">
        <label for="key">
            Key strength
        </label>
        <keygen name="requestkey" id="key" />
    </div>
  <input type="submit">
</form>
</div>


<div class="navbar navbar-inverse navbar-fixed-bottom">
    <div class="container">
        <a ng-click="about()"
            class="navbar-text pull-left">
            privacyIDEA
            {{ privacyideaVersionNumber }}
        </a>
        <a class="navbar-btn btn btn-success pull-right"
           href="https://netknights.it/leistungen/support/"
           target="_external">
            Support
        </a>
        <a class="navbar-text pull-right"
                href="https://groups.google.com/forum/#!forum/privacyidea"
                target="_external">
            privacyIDEA google group</a>
        <a class="navbar-text pull-right"
           ng-show="privacyideaVersionNumber"
                href="http://privacyidea.readthedocs.org/en/v{{ privacyideaVersionNumber }}"
                target="documentation">
            Online Documentation
        </a>
    </div>
</div>

{% include 'footer.html' %}
