
<div class="well" ng-controller="configController">
    <nav class="navbar navbar-default">
    <div class="container">
        <button type="button" class="navbar-toggle"
                data-toggle="collapse" data-target=".myConfigNavbar"
                aria-expanded="false" aria-controls="navbar">
            <span class="sr-only">Toggle navigation</span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
        </button>

        <div class="navbar-collapse collapse myConfigNavbar">
            <ul class="nav navbar-nav">
                <li ng-class="{active: $state.includes('config.system')}">
                    <a ui-sref="config.system">
                    <span class="glyphicon glyphicon-briefcase"
                          aria-hidden="true"></span>
                    <span translate>System</span></a></li>
                <li ng-class="{active: $state.includes('config.policies')}">
                    <a ui-sref="config.policies">
                    <span class="glyphicon glyphicon-hand-up"
                          aria-hidden="true"></span>
                    <span translate>Policies</span></a></li>
                <li ng-class="{active: $state.includes('config.events')}">
                    <a ui-sref="config.events">
                    <span class="glyphicon glyphicon-flag"
                          aria-hidden="true"></span>
                    <span translate>Events</span></a></li>
                <li ng-class="{active: $state.includes('config.tokens')}">
                    <a ui-sref="config.tokens">
                    <span class="glyphicon glyphicon-phone"
                          aria-hidden="true"></span>
                    <span translate>Tokens</span></a></li>
                <li ng-class="{active: $state.includes('config.mresolvers')}">
                    <a ui-sref="config.mresolvers">
                    <span class="glyphicon glyphicon-tasks"
                          aria-hidden="true"></span>
                        <span translate>Machines</span>
                    </a>
                </li>
                <li ng-class="{active: $state.includes('config.resolvers')}">
                    <a ui-sref="config.resolvers">
                        <span class="glyphicon glyphicon-user"
                          aria-hidden="true"></span>
                        <span translate>Users</span>
                    </a>
                </li>
                <li ng-class="{active: $state.includes('config.realms')}">
                    <a ui-sref="config.realms">
                        <span class="glyphicon glyphicon-globe"
                          aria-hidden="true"></span>
                        <span translate>Realms</span>
                    </a>
                </li>
                <li ng-class="{active: $state.includes('config.caconnectors')}">
                    <a ui-sref="config.caconnectors">
                        <span class="glyphicon glyphicon-bookmark"
                          aria-hidden="true"></span>
                        <span translate>CAs</span>
                    </a>
                </li>
            </ul>

        </div>
        <!--/.nav-collapse -->
    </div>
    </nav>

    <div ui-view class="slide"></div>

</div>
