<div class="well">
    <div class="alert alert-info alert-dismissible" role="alert">
            <button type="button" class="close" data-dismiss="alert"
                    aria-label="Close">
                <span aria-hidden="true">&times;</span></button>
            <translate>
                If you forgot your password, you may set a new one.
            </translate>
        </div>
    <div class="container">

         <form name="formRecovery" class="form-signin" role="form" novalidate>

            <h2 class="form-signin-heading" translate>Reset Password
            </h2>
                <div class="form-group"
                     ng-class="{'has-error': !newUser.user }">
                <label for="username" translate>Username</label>
                <input name="username" id="loginname"
                       class="form-control"
                       ng-model="newUser.user" required
                       placeholder="{{ 'Username'|translate }}"
                       autofocus/>
                    </div>

             <div class="form-group"
                  ng-class="{'has-error': !newUser.email }">
                <label for="email" translate>Email</label>
                <input name="email" id="email"
                       class="form-control"
                       ng-model="newUser.email" ng-required="true"
                       placeholder="{{ 'Email'|translate }}"
                       />
                 </div>


                <div class="text-center">
                    <button ng-click="sendRecoveryCode()"
                            ng-disabled="formRecovery.$invalid"
                            class="btn btn-primary btn-block"
                            translate>Send Reset Code
                    </button>
                </div>

            </form>
    </div>
</div>
