<p translate>
    The certificate was enrolled successfully.
</p>
<p>
    <a download="{{ enrolledToken.serial }}.pem"
       ng-href="{{ certificateBlob }}">
        <span class="glyphicon glyphicon-bookmark"
                          aria-hidden="true"></span>
        <span translate>Download the certificate</span></a>.
</p>
<!-- If the keys were generated on the server,
     we can download a PKCS12 file -->
<p ng-show="enrolledToken.pkcs12">
    <a download="{{ enrolledToken.serial }}.p12"
       ng-href="{{ pkcs12Blob }}">
        <span class="glyphicon glyphicon-bookmark"
                          aria-hidden="true"></span>
        <span translate>Download the PKCS12 file</span></a>.
</p>
<pre>
{{ enrolledToken.certificate }}
</pre>
