{% include 'header.html' %}

<div class="navbar navbar-default navbar-fixed-top">
    <div class="container">
        <a class="navbar-brand" href="#">privacyIDEA Certificate</a>

        <ul class="nav navbar-nav navbar-right">
            <li>
                <button onclick="window.close()"
                        class="btn btn-sm btn-warning">
                    Close Window
                </button>
            </li>
        </ul>
    </div>
</div>

<!----------------- main -------------------------->

<div class="container">

<p>
The certificate with token serial {{ serial }} for user {{ username }}
    ({{ role }}) was    enrolled    successfully.
</p>

<a href="data:application/x-x509-user-cert;base64,{{ cert_pem }}">Install
    Certificate to Browser</a>

<div class="well">
    <pre>{{ certificate }}
    </pre>
</div>

</div>

<!------------------ footer ------------------------------->

<div class="navbar navbar-inverse navbar-fixed-bottom">
    <div class="container">
        <a ng-click="about()"
            class="navbar-text pull-left">
            privacyIDEA
            {{ privacyideaVersionNumber }}
        </a>
        <a class="navbar-btn btn btn-success pull-right"
           href="https://netknights.it/leistungen/support/"
           target="_external">
            Support
        </a>
        <a class="navbar-text pull-right"
                href="https://groups.google.com/forum/#!forum/privacyidea"
                target="_external">
            privacyIDEA google group</a>
        <a class="navbar-text pull-right"
           ng-show="privacyideaVersionNumber"
                href="http://privacyidea.readthedocs.org/en/v{{ privacyideaVersionNumber }}"
                target="documentation">
            Online Documentation
        </a>
    </div>
</div>

{% include 'footer.html' %}
