.papertoken table {
    border-collapse: collapse;
    border-spacing: 0px 0px !important;
}

.papertoken th {
    border:1px dotted darkgray;
    background-color: lightgrey;
    padding: 10px;
}

.papertoken tr {
    background-color: whitesmoke;
}


.papertoken td {
    border-bottom:1px dotted orange;
    border-left: 1px solid lightgrey;
    border-right: 1px solid lightgrey;
    padding: 10px;
    border-collapse: collapse;
}

