<div class="row" ng-controller="configController">
    <div class="col-lg-3">
        <div class="pa-sidebar panel">
            <ul class="nav nav-pills nav-stacked">
                <li role="presentation"
                    ng-class="{ active: $state.includes('config.realms.list')}">
                    <a ui-sref="config.realms.list"
                            translate>All Realms</a></li>
                <li><a ng-click="clearDefaultRealm()"
                        translate>Clear default realm</a></li>
            </ul>
        </div>

    </div>
    <div ui-view class="col-lg-9 slide panel">
        <!-- load the realms.list state by default-->
        <div ng-init="$state.go('config.realms.list')">
        </div>
    </div>
</div>

