<!DOCTYPE html>
<html ng-app="privacyideaApp">
<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta name="description" content="">
    <meta name="author" content="Cornelius Kölbel" >
    <link rel="icon" href="../../favicon.ico">
    <!-- Custom styles for this template -->
    <link href="{{ instance }}/static/css/signin.css" rel="stylesheet">
    <link href="{{ instance }}/static/contrib/css/animate.css" rel="stylesheet">
    <link href="{{ instance }}/static/css/navbar-fixed-top.css" rel="stylesheet">
    <link href="{{ instance }}/static/contrib/css/angular-multi-select.css"
          rel="stylesheet"/>
    <link href="{{ instance }}/static/contrib/css/angular-inform.css"
          rel="stylesheet"/>
    <link href="{{ instance }}/static/contrib/css/hotkeys.min.css"
          rel="stylesheet"/>
    <link href="{{ instance }}/static/contrib/css/bootstrap.css" rel="stylesheet">
    <link href="{{ instance }}{{ theme }}" rel="stylesheet"/>

    <script type="application/javascript">
        // Pass the browser Language to our Javascript code
        var browserLanguage = "{{ browser_lang }}";
    </script>

</head>
<body ng-controller="mainController">
