/*
Taken from the google login page
*/

#title {
    text-align: center;
}
.circle {
    position: absolute;
    z-index: 101;
    height: 96px;
    width: 96px;
    border-radius: 50%;
    opacity: 0.99;
    overflow: hidden;
    background-repeat: no-repeat;
    background-position: center center;
}

.circle-mask {
    display: block;
    height: 96px;
    width: 96px;
    overflow: hidden;
    border-radius: 50%;
    margin-left: auto;
    margin-right: auto;
    z-index: 100;
    margin-bottom: 10px;
    background-size: 96px;
    background-repeat: no-repeat;
    background-image: url(https://ssl.gstatic.com/accounts/ui/avatar_2x.png);
    -webkit-transition: opacity 0.075s;
    -moz-transition: opacity 0.075s;
    -ms-transition: opacity 0.075s;
    -o-transition: opacity 0.075s;
    transition: opacity 0.075s;
}


.input-wrapper {
    position: relative;
}

.slide-out {
    -webkit-transform: translate3d(0, 0, 0);
    -moz-transform: translate3d(0, 0, 0);
    -ms-transform: translate3d(0, 0, 0);
    -o-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0);
}

.slide-in, .slide-out {
    display: block;
    -webkit-transition-property: -webkit-transform, opacity;
    -moz-transition-property: -moz-transform, opacity;
    -ms-transition-property: -ms-transform, opacity;
    -o-transition-property: -o-transform, opacity;
    transition-property: transform, opacity;
    -webkit-transition-duration: 0.1s;
    -moz-transition-duration: 0.1s;
    -ms-transition-duration: 0.1s;
    -o-transition-duration: 0.1s;
    transition-duration: 0.1s;
    -webkit-transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    -moz-transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    -ms-transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    -o-transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
    transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1);
}

.main-content {
    height: 450px;
}

div {
    display: block;
}

html, body {
    font-family: Arial, sans-serif;
    background: #fff;
    margin: 0;
    padding: 0;
    border: 0;
    position: absolute;
    height: 100%;
    min-width: 100%;
    font-size: 13px;
    color: #404040;
    direction: ltr;
    -webkit-text-size-adjust: none;
}

.signin-card {
    min-height: 500px;
    position: relative;
    overflow: hidden;
    width: 274px;
    padding: 40px 40px;
}

.card {
    margin-bottom: 0;
    background-color: #f7f7f7;
    padding: 20px 25px 30px;
    margin: 0 auto 25px;
    width: 304px;
    -moz-border-radius: 2px;
    -webkit-border-radius: 2px;
    border-radius: 2px;
    -moz-box-shadow: 0px 2px 2px rgba(0, 0, 0, 0.3);
    -webkit-box-shadow: 0px 2px 2px rgba(0, 0, 0, 0.3);
    box-shadow: 0px 2px 2px rgba(0, 0, 0, 0.3);
}

.rc-button,
.rc-button:visited {
    display: inline-block;
    min-width: 46px;
    text-align: center;
    color: #444;
    font-size: 14px;
    font-weight: 700;
    height: 36px;
    padding: 0 8px;
    line-height: 36px;
    -moz-border-radius: 3px;
    -webkit-border-radius: 3px;
    border-radius: 3px;
    -o-transition: all 0.218s;
    -moz-transition: all 0.218s;
    -webkit-transition: all 0.218s;
    transition: all 0.218s;
    border: 1px solid #dcdcdc;
    background-color: #f5f5f5;
    background-image: -webkit-linear-gradient(top,#f5f5f5,#f1f1f1);
    background-image: -moz-linear-gradient(top,#f5f5f5,#f1f1f1);
    background-image: -ms-linear-gradient(top,#f5f5f5,#f1f1f1);
    background-image: -o-linear-gradient(top,#f5f5f5,#f1f1f1);
    background-image: linear-gradient(top,#f5f5f5,#f1f1f1);
    -o-transition: none;
    -moz-user-select: none;
    -webkit-user-select: none;
    user-select: none;
    cursor: default;
}
.card .rc-button {
    width: 100%;
    padding: 0;
}

.rc-button-submit,
.rc-button-submit:visited {
    border: 1px solid #3079ed;
    color: #fff;
    text-shadow: 0 1px rgba(0,0,0,0.1);
    background-color: #4d90fe;
    background-image: -webkit-linear-gradient(top,#4d90fe,#4787ed);
    background-image: -moz-linear-gradient(top,#4d90fe,#4787ed);
    background-image: -ms-linear-gradient(top,#4d90fe,#4787ed);
    background-image: -o-linear-gradient(top,#4d90fe,#4787ed);
    background-image: linear-gradient(top,#4d90fe,#4787ed);
}
.rc-button-submit:hover {
    border: 1px solid #2f5bb7;
    color: #fff;
    text-shadow: 0 1px rgba(0,0,0,0.3);
    background-color: #357ae8;
    background-image: -webkit-linear-gradient(top,#4d90fe,#357ae8);
    background-image: -moz-linear-gradient(top,#4d90fe,#357ae8);
    background-image: -ms-linear-gradient(top,#4d90fe,#357ae8);
    background-image: -o-linear-gradient(top,#4d90fe,#357ae8);
    background-image: linear-gradient(top,#4d90fe,#357ae8);
}
.rc-button-submit:active {
    background-color: #357ae8;
    background-image: -webkit-linear-gradient(top,#4d90fe,#357ae8);
    background-image: -moz-linear-gradient(top,#4d90fe,#357ae8);
    background-image: -ms-linear-gradient(top,#4d90fe,#357ae8);
    background-image: -o-linear-gradient(top,#4d90fe,#357ae8);
    background-image: linear-gradient(top,#4d90fe,#357ae8);
    -moz-box-shadow: inset 0 1px 2px rgba(0,0,0,0.3);
    -webkit-box-shadow: inset 0 1px 2px rgba(0,0,0,0.3);
    box-shadow: inset 0 1px 2px rgba(0,0,0,0.3);
}

input[type=email], .signin-card input[type=password], .signin-card input[type=text], .signin-card input[type=submit] {
    width: 100%;
    display: block;
    margin-bottom: 10px;
    z-index: 1;
    position: relative;
    -moz-box-sizing: border-box;
    -webkit-box-sizing: border-box;
    box-sizing: border-box;
    direction: ltr;
    height: 44px;
    font-size: 16px;
}

.btnaddonright {
    widht: 50px;
}
.btnaddonright img{
    width: 50px;
}
