<p class="help-block" translate>
The registration token is a code, that the user can use to authenticate once!
    After using this code to login, the registration token is deleted and can
    not be used anymore.
</p>
<p class="help-block" translate>
    This is used, so that the user can enroll a token on his own, after
    logging in for the first time.
</p>
