
<div class="well">
    <nav class="navbar navbar-default">
    <div class="container">
        <button type="button" class="navbar-toggle"
                data-toggle="collapse" data-target=".myConfigNavbar"
                aria-expanded="false" aria-controls="navbar">
            <span class="sr-only">Toggle navigation</span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
        </button>

        <div class="navbar-collapse collapse myConfigNavbar">
            <ul class="nav navbar-nav">
                <li ng-class="{active: $state.includes('audit.log')}">
                    <a ui-sref="audit.log">
                    <span class="glyphicon glyphicon-search"
                          aria-hidden="true"></span>
                    <span translate>Log</span></a></li>
                <li ng-class="{active: $state.includes('audit.stats')}"
                    ng-show="loggedInUser.role === 'admin'">
                    <a ui-sref="audit.stats">
                    <span class="glyphicon glyphicon-stats"
                          aria-hidden="true"></span>
                    <span translate>Statistics</span></a></li>
            </ul>

        </div>
        <!--/.nav-collapse -->
    </div>
    </nav>

    <div ui-view class="slide"></div>

</div>
