<a href="" class="btn btn-primary" ng-click="downloadCSV()">
    <span class="glyphicon glyphicon-download"></span>
    <span translate>Download</span></a>
<div class="progress" ng-show="downloadProgress">
  <div class="progress-bar progress-bar-striped active" role="progressbar"
       aria-valuenow="100" aria-valuemin="0" aria-valuemax="100" style="width:
       100%">
    <span class="sr-only" translate>45% Complete</span>
  </div>
</div>
