<div>
    <table class="table table-bordered table-responsive table-striped">
        <thead>
        <tr>
            <th translate>Resolver name</th>
            <th translate>type</th>
        </tr>
        </thead>
        <tbody>
        <tr ng-repeat="resolver in resolvers">
            <td><a ui-sref="config.resolvers.edit{{ resolver.type }}
                    ({resolvername:resolver.resolvername})">
                        {{ resolver.resolvername }}</a>
            </td>
            <td>
                {{ resolver.type }}
            </td>
            <td>
                <button class="btn btn-sm btn-primary"
                        ng-click="editResolver(resolver.resolvername, resolver.type)"
                        translate>Edit</button>
                <button class="btn btn-sm btn-danger"
                        ng-click="delResolver(resolver.resolvername)">
                    <span class="glyphicon glyphicon-trash"></span>
                    <span translate>Delete</span>
                </button>
            </td>
        </tr>
        </tbody>
    </table>
</div>
