<?php
if(!empty($this->data['htmlinject']['htmlContentPost'])) {
	foreach($this->data['htmlinject']['htmlContentPost'] AS $c) {
		echo $c;
	}

}
?>
				</div> <!-- signin-card -->
			</div> <!-- main content -->
		</div> <!-- container -->
	</div> <!-- wrapper -->
</div>
</body>
</html>
