<div class="row" ng-controller="configController">
    <div class="col-lg-3">
        <div class="pa-sidebar panel">
            <ul class="nav nav-pills nav-stacked">
                <li role="presentation"
                    ng-class="{ active: $state.includes('config.resolvers.list')}">
                    <a ui-sref="config.resolvers.list"
                            translate>All Resolvers</a></li>
                <li role="presentation"
                    ng-class="{ active: $state.includes('config.resolvers.edit')}"
                        ng-show="$state.includes('config.resolvers.edit')">
                    <a ui-sref="config.resolvers.edit"
                            translate>Edit Resolver {{ resolvername }}</a></li>

                <li class="nav nav-pills nav-header"
                        translate>New Resolvers</li>
                <!-- Here we can get all resolvers and iterated over
                     the resolver types.
                -->
                <li ng-repeat="resolvertype in availableResolverTypes"
                        role="presentation"
                    ng-class="{active:
                    $state.includes('config.resolvers.add'+resolvertype) }">
                    <a ui-sref="config.resolvers.add{{ resolvertype }}"
                            translate>
                        New {{ resolvertype }}</a>
                </li>
            </ul>
        </div>

    </div>
    <div ui-view class="col-lg-9 slide panel">
        <!-- load the resolvers.list state by default
        Das geht nicht, weil er dann auch auf den Unterseiten zurückspringt.
        <div ng-init="$state.go('config.resolvers.list')">
        </div>
        -->
    </div>
</div>

